// This work is licensed under the Creative Commons Attribution-ShareAlike 3.0 Unported License.
// To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/3.0/ or send a
// letter to Creative Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.

//    Persistence of Vision Ray Tracer version 3.8 Include File. (povr version ONLY)
//    File: strings.inc
//    Last updated: 22 June 2020
//    Description: This file contains various macros for manipulating strings.

#ifndef(STRINGS_INC_TEMP)
#declare STRINGS_INC_TEMP = version;
#version 3.8;

#ifdef(View_POV_Include_Stack)
   #debug "including strings.inc\n"
#end

// CRGBFTStr macro

// Converts a color to a string, with filter and transmit

#macro CRGBFTStr(C, MinLen, Padding)
    concat( "color rgbft < ", str(C.red, MinLen, Padding),
                        ", ", str(C.green, MinLen, Padding),
                        ", ", str(C.blue, MinLen, Padding),
                        ", ", str(C.filter, MinLen, Padding),
                        ", ", str(C.transmit, MinLen, Padding), ">")
#end

// CRGBStr macro

// Converts a color to a string, without filter and transmit

#macro CRGBStr(C, MinLen, Padding)
    concat( "color rgb <   ", str(C.red, MinLen, Padding),
                        ", ", str(C.green, MinLen, Padding),
                        ", ", str(C.blue, MinLen, Padding), ">")
#end

// Parse_String macro

// Parse a string you create (i.e., = concat(...)) - while
// parsing.  Use carefully. This macro long a hack using a file
// which can collide by name across invocations of povray or
// during multiple simultaneous povray commands.

// In povr the macro changed so users can pass an optional
// filename less likely to collide the default name. The optional
// argument forces at least a comma as in:

//   Parse_String("THING1",)

// even when empty or an undefined identifier. The usual way would
// be:

//   Parse_String("THING1","/tmp/wibbly_wobbly_parse_stringyWingy")

// Further, in opening and closing a file it is also inefficient
// if done many times. Specifying a path and file name on a
// ramdisk will help. Often it's better to use the same "include
// defined strings for parsing" idea by writing many 'to be parsed
// strings' to a file and then include that single file by a name
// which you know and control.

#macro Parse_String(_string, optional _filename)
    #ifndef(local._filename)
        #local Filename = "parse_string.tmp"; // File name default
    #else    // Necessary if #local _<lowercase> illegal.
        #local Filename = _filename;
    #end
    #fopen _fileout Filename write
    #write(_fileout,_string)
    #fclose _fileout
    #include Filename
#end

//just a shortcut for using str() with system-defined precision and length.
#macro Str(A)
    str(A, 0,-1)
#end

//just a shortcut for using vstr()
//"Str" is with capitalized "S" to match the Str() macro
//above which also has system-defined precision and length.
#macro VStr2D(V)
    concat("<", vstr(2, V, ",", 0,-1), ">")
#end
#macro VStr(V)
    concat("<", vstr(3, V, ",", 0,-1), ">")
#end

//Another shortcut for using vstr(),
//this one with user specified precision and length.
//"str" is with non-capitalized "s" to match the str() function
//in POV-Ray which also has user-specified precision and length.
#macro Vstr2D(V,L,P)
    concat("<", vstr(2, V, ",", L,P), ">")
#end
#macro Vstr(V,L,P)
    concat("<", vstr(3, V, ",", L,P), ">")
#end

//Macros used to generate strings with mesh syntax.
//Can be used with the IO features to write a mesh to an external file.
#macro Triangle_Str(A, B, C)
    concat("triangle {<",   vstr(3, A, ",", 0,-1), ">,<",
                            vstr(3, B, ",", 0,-1), ">,<",
                            vstr(3, C, ",", 0,-1), ">}")
#end
#macro Smooth_Triangle_Str(A, NA, B, NB, C, NC)
    concat("smooth_triangle {<",    vstr(3, A, ",", 0,-1), ">,<",
                                    vstr(3, NA, ",", 0,-1), ">,<",
                                    vstr(3, B, ",", 0,-1), ">,<",
                                    vstr(3, NB, ",", 0,-1), ">,<",
                                    vstr(3, C, ",", 0,-1), ">,<",
                                    vstr(3, NC, ",", 0,-1), ">}")
#end

//================================= End render time declarations ===================================

//--- Self testing:  povr +mv3.8 +i<this_file>
#if(input_file_name="strings.inc")

// Add scene(s) here

#end // End self testing

#version STRINGS_INC_TEMP;
#end//strings.inc

