//******************************************************************************
///
/// @file core/math/jitter.h
///
/// @todo   What's in here?
///
/// @copyright
/// @parblock
///
/// Persistence of Vision Ray Tracer ('POV-Ray') version 3.8.
/// Copyright 1991-2019 Persistence of Vision Raytracer Pty. Ltd.
///
/// POV-Ray is free software: you can redistribute it and/or modify
/// it under the terms of the GNU Affero General Public License as
/// published by the Free Software Foundation, either version 3 of the
/// License, or (at your option) any later version.
///
/// POV-Ray is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU Affero General Public License for more details.
///
/// You should have received a copy of the GNU Affero General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.
///
/// ----------------------------------------------------------------------------
///
/// POV-Ray is based on the popular DKB raytracer version 2.12.
/// DKBTrace was originally written by David K. Buck.
/// DKBTrace Ver 2.0-2.12 were written by David K. Buck & Aaron A. Collins.
///
/// @endparblock
///
//******************************************************************************

#ifndef POVRAY_CORE_JITTER_H
#define POVRAY_CORE_JITTER_H

// Module config header file must be the first file included within POV-Ray unit header files
#include "core/configcore.h"

// C++ variants of C standard header files
// C++ standard header files
//  (none at the moment)

// POV-Ray header files (base module)
// POV-Ray header files (core module)
//  (none at the moment)

namespace pov
{

//##############################################################################
///
/// @defgroup PovCoreMathJitter Pseudorandom Jitter
/// @ingroup PovCoreMath
///
/// @{

#ifdef DYNAMIC_HASHTABLE
extern unsigned short *hashTable; // GLOBAL VARIABLE
#else
alignas(16) extern unsigned short hashTable[]; // GLOBAL VARIABLE
#endif

extern const float JitterTable[]; // GLOBAL VARIABLE  (Values range -0.5 to 0.5)

// These two int input and the two double input versions are not currently used.
//
// inline DBL Jitter2d(int x, int y)
// {
//     return JitterTable[int(hashTable[int(hashTable[int(x & 0xfff)] ^ y) & 0xfff]) & 0xff];
// }
//
// inline void Jitter2d(int x, int y, DBL& jx, DBL& jy)
// {
//     jx = JitterTable[int(hashTable[int(hashTable[int((x + 1021) & 0xfff)] ^ (y + 1019)) & 0xfff]) & 0xff]; // Note that 1019 and 1021 are prime! [trf]
//     jy = JitterTable[int(hashTable[int(hashTable[int((x + 1019) & 0xfff)] ^ (y + 1021)) & 0xfff]) & 0xff]; // Note that 1019 and 1021 are prime! [trf]
// }
//
// inline DBL Jitter2d(DBL x, DBL y)
// {
//     return JitterTable[int(hashTable[int(hashTable[(int(x * 1021.0) & 0xfff)] ^ int(y * 1021.0)) & 0xfff]) & 0xff]; // Note that 1021 is prime! [trf]
// }
//

// Version below for focal blur only. TODO change?  This was the version used
// to AA methods 1 and 2 jitter in v3.7 and v3.8, but it's no better than no
// jitter and it bunches badly over smaller regions within an image and x,y
// pairs as little as 8% random in images as small as 300x300 due straight
// calls. Focal blur usage better due randomizing x,y inputs ahead of calls.

inline void Jitter2d_fclblr(DBL x, DBL y, DBL& jx, DBL& jy)
{
    jx = JitterTable[int(hashTable[int(hashTable[(int(x * 1021.0) & 0xfff)] ^ int(y * 1019.0)) & 0xfff]) & 0xff]; // Note that 1019 and 1021 are prime! [trf]
    jy = JitterTable[int(hashTable[int(hashTable[(int(x * 1019.0) & 0xfff)] ^ int(y * 1021.0)) & 0xfff]) & 0xff]; // Note that 1019 and 1021 are prime! [trf]
}


// Below used in povr with +j for truly random jitter over that above used in v3.7 and v3.8.

inline void Jitter2d(DBL x, DBL y, DBL& jx, DBL& jy)
{
    jx = -0.5f + static_cast<double>(rand()) / (static_cast<double>(RAND_MAX/((0.5)-(-0.5f))));
    jy = -0.5f + static_cast<double>(rand()) / (static_cast<double>(RAND_MAX/((0.5)-(-0.5f))));
}

/// @}
///
//##############################################################################

}
// end of namespace pov

#endif // POVRAY_CORE_JITTER_H
